import json
import logging


class Cache:
    _CACHE_BASE_PATH = '/tmp'

    _LOGGER = logging.getLogger()
    _LOGGER.setLevel(logging.INFO)

    def __init__(self, name):
        self._cache_path = f"{self._CACHE_BASE_PATH}/{name}.json"
        try:
            with open(self._cache_path) as cache_file:
                self.cache = json.load(cache_file)
        except IOError:
            self._LOGGER.info("No cache available. Creating empty one.")
            self.cache = {}

    @property
    def cache_path(self):
        return self._cache_path

    def get_all(self):
        return self.cache

    def set_all(self, keys_with_values):
        for key in keys_with_values:
            self.set(key, keys_with_values[key])

    def get(self, key):
        return self.cache.get(key, None)

    def set(self, key, value):
        self.cache[key] = value
        with open(self._cache_path, 'w') as cache_file:
            json.dump(self.cache, cache_file)

    def delete_all(self):
        self.cache = {}
        with open(self._cache_path, 'w') as cache_file:
            json.dump(self.cache, cache_file)



import json
import logging

from botocore.exceptions import NoCredentialsError
from time import time
from typing import Dict

from cached_secrets_manager.cache import Cache
from cached_secrets_manager.no_secrets_found_error import NoSecretsFoundError
from cached_secrets_manager.secrets_manager import SecretsManager


class CachedSecretsManager:
    _VALID_UNTIL_KEY = 'valid_until'

    _LOGGER = logging.getLogger()
    _LOGGER.setLevel(logging.INFO)

    def __init__(self, cache_path: str, secret_name: str, region_name: str):
        self._secrets_cache: Cache = Cache(cache_path)
        self._secrets_manager: SecretsManager = SecretsManager(secret_name, region_name)

    def get(self, minutes_to_live: int = 60) -> Dict:
        """
        This method tries to retrieve a secret from cache first and falls back to secret manager otherwise.
        :return: secret of throws exception
        """
        try:
            secrets: Dict = self._get_secrets_from_cache()
            current_time = int(round(time()))
            valid_until = int(secrets.get(self._VALID_UNTIL_KEY, 0))
            if current_time > valid_until:
                self._LOGGER.info(f"Cache needs to be invalidated (Current time: {current_time}, valid until: {valid_until}).")
                self.reset_cache()
                raise NoSecretsFoundError
        except NoSecretsFoundError as error:
            self._LOGGER.info(f"No secrets found in cache {self._secrets_cache.cache_path}. Trying through secrets manager.")
            secrets = self._get_secrets_from_secrets_manager()
            valid_until = int(round(time())) + minutes_to_live*60
            secrets[self._VALID_UNTIL_KEY] = valid_until
            self._set_secrets_in_cache(secrets)
        secrets.pop(self._VALID_UNTIL_KEY, None)
        return secrets

    def set(self, secrets: Dict):
        self._secrets_manager.update(json.dumps(secrets))
        self.reset_cache()

    def reset_cache(self):
        self._secrets_cache.delete_all()

    def _get_secrets_from_cache(self):
        secrets = self._secrets_cache.get_all()
        if not secrets:
            raise NoSecretsFoundError("No secrets found in cache.")
        for secret_value in secrets.values():
            if not secret_value:
                raise NoSecretsFoundError("No secrets found in cache.")
        self._LOGGER.info('Found secrets in cache.')
        return secrets

    def _set_secrets_in_cache(self, secrets):
        self._secrets_cache.set_all(secrets)
        self._LOGGER.info('Set secrets in cache.')

    def _get_secrets_from_secrets_manager(self):
        self._LOGGER.info('Retrieving secrets from secrets manager.')
        try:
            secrets = json.loads(self._secrets_manager.get())
        except NoCredentialsError:
            raise NoSecretsFoundError(f"Secret missing in secrets manager.")
        if not secrets:
            raise NoSecretsFoundError(f"No secrets found in secret.")
        for secret_value in secrets.values():
            if not secret_value:
                raise NoSecretsFoundError(f"Incomplete secrets found through secrets manager.")
        return secrets



import boto3
import base64

from botocore.exceptions import ClientError


class SecretsManager:
    ERROR_KEY = 'Error'
    CODE_KEY = 'Code'

    def __init__(self, secret_name, region_name):
        if not secret_name or not region_name:
            raise Exception("Secret name or region name need to be provided.")
        self.secret_name = secret_name
        self.client = boto3.session.Session().client(
            service_name='secretsmanager',
            region_name=region_name
        )

    def get(self):
        try:
            get_secret_value_response = self.client.get_secret_value(SecretId=self.secret_name)
        except ClientError as e:
            if 'DecryptionFailureException' == e.response[self.ERROR_KEY][self.CODE_KEY]:
                # Secrets Manager can't decrypt the protected secret text using the provided KMS key.
                raise e
            elif 'InternalServiceErrorException' == e.response[self.ERROR_KEY][self.CODE_KEY]:
                # An error occurred on the server side.
                raise e
            elif 'InvalidParameterException' == e.response[self.ERROR_KEY][self.CODE_KEY]:
                # You provided an invalid value for a parameter.
                raise e
            elif 'InvalidRequestException' == e.response[self.ERROR_KEY][self.CODE_KEY]:
                # You provided a parameter value that is not valid for the current state of the resource.
                raise e
            elif 'ResourceNotFoundException' == e.response[self.ERROR_KEY][self.CODE_KEY]:
                # We can't find the resource that you asked for.
                raise e
        else:
            # Decrypts secret using the associated KMS CMK. Depending on whether the secret is a string or binary, one
            # of these fields will be populated.
            if 'SecretString' in get_secret_value_response:
                return get_secret_value_response['SecretString']
            else:
                return base64.b64decode(get_secret_value_response['SecretBinary'])

    def update(self, data):
        return self.client.update_secret(SecretId=self.secret_name, SecretString=data)
